searchData={"items":[{"type":"module","title":"ExPression","doc":"Define and evaluate expressions in runtime in your Elixir project.","ref":"ExPression.html"},{"type":"module","title":"Features - ExPression","doc":"* JSON support - expressions support all JSON types with it's stanrad syntax\n* Python-like operators and standard functions\n* Extend expressions by providing Elixir module with functions that you want to use.\n* Safe evaluation without acces to other Elixir modules.","ref":"ExPression.html#module-features"},{"type":"function","title":"ExPression.eval/2","doc":"Evalate expression in string or AST format.","ref":"ExPression.html#eval/2"},{"type":"function","title":"Options - ExPression.eval/2","doc":"* `:bindings` - map variable names and values.\n* `:functions_module` - module with functions that will be accessible from expressions.","ref":"ExPression.html#eval/2-options"},{"type":"function","title":"Examples - ExPression.eval/2","doc":"iex> eval(\"1 + 0.5\")\n    {:ok, 1.5}\n    iex> eval(\"div(x, y)\", bindings: %{\"x\" => 5, \"y\" => 2}, functions_module: Kernel)\n    {:ok, 2}\n    iex> eval(~s/{\"1\": \"en\", \"2\": \"fr\"}[str(int_code)]/, bindings: %{\"int_code\" => 1})\n    {:ok, \"en\"}\n    iex> eval(\"not true or false or 1 == 1\")\n    {:ok, true}\n    iex> eval(\"exit(self())\")\n    {:error, %ExPression.Error{name: \"UndefinedFunctionError\", message: \"Function 'self/0' was referenced, but was not defined\", data: %{function: :self}}}","ref":"ExPression.html#eval/2-examples"},{"type":"function","title":"ExPression.eval_ast/2","doc":"","ref":"ExPression.html#eval_ast/2"},{"type":"function","title":"ExPression.parse/1","doc":"Parse expression in string format into AST format.\n\nThis can be used for optimizations: to parse expression once and evaluate AST many times.","ref":"ExPression.html#parse/1"},{"type":"type","title":"ExPression.ast/0","doc":"","ref":"ExPression.html#t:ast/0"},{"type":"module","title":"ExPression.Error","doc":"Error that occurred while parsing or evaluating expression.","ref":"ExPression.Error.html"},{"type":"function","title":"ExPression.Error.new/3","doc":"","ref":"ExPression.Error.html#new/3"},{"type":"type","title":"ExPression.Error.t/0","doc":"","ref":"ExPression.Error.html#t:t/0"},{"type":"module","title":"ExPression.Interpreting","doc":"Evaluating AST","ref":"ExPression.Interpreting.html"},{"type":"function","title":"ExPression.Interpreting.eval/3","doc":"","ref":"ExPression.Interpreting.html#eval/3"},{"type":"module","title":"ExPression.Parsing","doc":"Parsing expressions in strings format with convertion to AST format","ref":"ExPression.Parsing.html"},{"type":"function","title":"ExPression.Parsing.parse/1","doc":"","ref":"ExPression.Parsing.html#parse/1"},{"type":"module","title":"ExPression.Parsing.Grammar","doc":"Expressions formal language grammar definition","ref":"ExPression.Parsing.Grammar.html"},{"type":"function","title":"ExPression.Parsing.Grammar.peg/0","doc":"","ref":"ExPression.Parsing.Grammar.html#peg/0"},{"type":"module","title":"ExPression.StandardLib","doc":"Standard library of functinos for ExPressions\n\nYou can override or add to standard functinos by providing a module to ExPression.","ref":"ExPression.StandardLib.html"},{"type":"function","title":"ExPression.StandardLib.str/1","doc":"","ref":"ExPression.StandardLib.html#str/1"},{"type":"extras","title":"ExPression","doc":"# ExPression\nEval user input expressions in your Elixir project.","ref":"readme.html"},{"type":"extras","title":"Installation - ExPression","doc":"The package can be installed by adding `ex_pression` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_pression, \"~> 0.2.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"SAFE - ExPression","doc":"Safe evaluation without acces to other Elixir modules.\n```elixir\nExPression.eval(\"exit(self())\")\n{:error, %ExPression.Error{name: \"UndefinedFunctionError\", message: \"Function 'self/0' was referenced, but was not defined\", data: %{function: :self}}}\n```","ref":"readme.html#safe"},{"type":"extras","title":"JSON - ExPression","doc":"Support for JSON syntax and data types.\n```elixir\niex> ExPression.eval(\"\"\"\n{\n  \"name\": \"ex_pression\",\n  \"deps\": [\"xpeg\"]\n}\n\"\"\")\n{:ok, %{\"name\" => \"ex_pression\", \"deps\" => [\"xpeg\"]}}\n```","ref":"readme.html#json"},{"type":"extras","title":"PYTHON - ExPression","doc":"Familiar python-like operators and standard functions.\n```elixir\niex> ExPression.eval(~s/{\"1\": \"en\", \"2\": \"fr\"}[str(int_code)]/, bindings: %{\"int_code\" => 1})\n{:ok, \"en\"}\n```","ref":"readme.html#python"},{"type":"extras","title":"EXTEND - ExPression","doc":"Extend expressions by providing Elixir module with functions that you want to use.\n```elixir\niex> ExPression.eval(\"div(5, 2)\", functions_module: Kernel)\n{:ok, 2}\n```","ref":"readme.html#extend"},{"type":"extras","title":"Features list - ExPression","doc":"- [x] JSON data types\n  - [x] Integer\n  - [x] Float\n  - [x] String\n  - [x] Array\n  - [x] Object\n- [x] Standrad library of functions\n- [x] Variables bindings\n- [x] Extending by providing modules with functions\n- [x] Boolean Operators\n  - [x] `and`, `or`, `not`\n  - [x] `==`, `!=`, `<`, `<=`, `>`, `>=`\n- [x] Math Operators\n  - [x] `+`, `-`, `*`, `/`\n  - [ ] `**` (power operator)\n- [x] Access Operators\n  - [x] Field access (`obj.field_name`)\n  - [x] Access array element by index (`[1, 2, 3][0]`)\n  - [x] Access object field in runtime (`{\"fr\": \"France\", \"en\": \"England\"}[country]`)\n- [x] Error handling\n  - [x] Syntax errors \n  - [x] Runtime errors\n- [ ] AST Validations\n  - [ ] Calling non existing functions\n  - [ ] Invalid data types","ref":"readme.html#features-list"},{"type":"extras","title":"Implementation - ExPression","doc":"String representation of expression is parsed into AST form. Parsing is done with PEG grammar parser [xpeg](https://github.com/zevv/xpeg). Grammar is defined in module `ExPression.Parsing.Grammar`.\nAST interpretation logic is written in plain `Elixir` in module `ExPression.Interpreting`.","ref":"readme.html#implementation"},{"type":"extras","title":"Contribution - ExPression","doc":"Feel free to make a pull request. All contributions are appreciated!","ref":"readme.html#contribution"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2024 Matvey Karpov\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown"}